package com.iconplus.smartproc.service.print;

import com.iconplus.smartproc.exception.BusinessException;
import com.iconplus.smartproc.helper.model.Pagination;
import com.iconplus.smartproc.helper.service.BaseService;
import com.iconplus.smartproc.model.projection.DrpView;
import com.iconplus.smartproc.model.request.PrintDrpRequest;
import com.iconplus.smartproc.model.response.GetListPrintDrpResponse;
import com.iconplus.smartproc.model.response.PrintDrpResponse;
import com.iconplus.smartproc.repository.DrpApprovalRepository;
import com.iconplus.smartproc.repository.DrpRepository;
import com.iconplus.smartproc.util.Constants;
import org.apache.commons.lang3.BooleanUtils;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
public class GetListPrintDrpService implements BaseService<PrintDrpRequest, GetListPrintDrpResponse> {

    private DrpRepository drpRepository;
    private DrpApprovalRepository drpApprovalRepository;
    public GetListPrintDrpService(DrpRepository drpRepository,
                                  DrpApprovalRepository drpApprovalRepository) {
        this.drpRepository = drpRepository;
        this.drpApprovalRepository = drpApprovalRepository;
    }


    @Override
    public GetListPrintDrpResponse execute(PrintDrpRequest input) {

        List<PrintDrpResponse> printDrpResponseList = new ArrayList<>();

        var drpViews = drpRepository.getListPrintDrp(input.getPageable());
        for (DrpView drpView : drpViews) {
            PrintDrpResponse printDrpResponse = PrintDrpResponse.builder()
                    .id(drpView.getId())
                    .tahun(drpView.getTahun())
                    .printDate(drpView.getPrintDate())
                    .build();

            var drpApprovalOptional = drpApprovalRepository.findByDrpIdAndLevelAndIsDeleteFalse(drpView.getId(), "DIREKTUR")
                    .orElseThrow(()-> new BusinessException(Constants.ERR_CODE_10001,
                            Constants.ERR_TTL_10001,
                            String.format(Constants.ERR_MSG_10001, "DRP Approvel", drpView.getId())));

            if (BooleanUtils.isTrue(drpView.getIsPrint())) {
                printDrpResponse.setStatus("Tercetak");
            } else if (drpApprovalOptional.getIsApprove() == null) {
                printDrpResponse.setStatus("Belum Dicetak");
            } else {
                printDrpResponse.setStatus("Belum Bisa Dicetak");
            }
            printDrpResponseList.add(printDrpResponse);
        }

        return GetListPrintDrpResponse.builder()
                .data(printDrpResponseList)
                .pagination(Pagination.builder()
                        .pageSize(input.getPageable().getPageSize())
                        .currentPage(input.getPageable().getPageNumber())
                        .totalPages(drpViews.getTotalPages())
                        .totalRecords(drpViews.getTotalElements())
                        .isFirstPage(drpViews.isFirst())
                        .isLastPage(drpViews.isLast())
                        .build())
                .build();
    }
}
